package de.rpgframework;

/**
 * <p>Das Framework kümmert sich um die Initialisierung des gesamten Spiels.
 * Dabei wird das Spielfenster und dessen Render-Panel erzeugt. Die
 * Initialisierung erfolgt über die Methode {@link Framework#init}.</p>
 *
 * <p>Damit von dieser Klasse nur eine Instanz angelegt werden kann, wird
 * das Singleton Design-Pattern verwendet. Dabei wird der Konstruktor privat
 * gemacht und mit Hilfe der {@link Framework#get}-Methode greift
 * man dann auf die statische Instanz der Klasse zu.
 *
 * @author Ren&eacute; Papenfu&szlig;
 */
public final class Framework {

    /**
     * Die Initialisierung dieser statischen Variable ist notwendig, damit das
     * das Singleton-Verhalten dieser Klasse möglichst sicher und ohne
     * Komplikationen mit Threads vonstatten geht.
     */
    private static Framework framework = new Framework();

    /**
     * In dieser Variablen wird eine Referenz auf ein {@link GameWindow}
     * gespeichert.
     */
    private GameWindow gameWindow;

    /**
     *
     */
    private Thread gameThread;

    /**
     * In screenWidth und screenHeight wird die Auflösung des Programmfensters
     * gespeichert, damit von überall aus ohne Probleme darauf zugegriffen
     * werden kann.
     */
    private int screenWidth, screenHeight;

    /**
     * Der Konstruktor dieser Klasse ist privat, damit kein Objekt von außen
     * angelegt werden kann.
     */
    private Framework() { }

    /**
     * Die Methode gibt die Referenz auf das Framework-Objekt zurück.
     *
     * @return Referenz auf die Framework-Instanz
     */
    public static Framework get() {
        return framework;
    }

    /**
     * Zur Initialisierung wird diese Methode verwendet. Dabei muss die
     * gewünschte Auflösung des Spiels angegeben werden. Diese Methode erzeugt
     * das Spielfenster und dessen Inhalt.
     *
     * @param screenWidth Auflösung in x-Richtung
     * @param screenHeight Auflösung in y-Richtung
     */
    public void init(int screenWidth, int screenHeight) {
        // Die Programmauflösung abspeichern
        this.screenWidth  = screenWidth;
        this.screenHeight = screenHeight;

        // Das Spielfenster initialisieren
        gameWindow = new GameWindow();

        // Den GameThread initialisieren
        gameThread = new Thread(new GameThread());
    }

    /**
     * Diese Methode startet des Programm. Es wird das Fenster angezeigt und
     * eventuelle Threads werden gestarted.
     */
    public void run() {
        // Anzeigen des Spielfensters
        gameWindow.setVisible(true);

        // Starten des GameThreads
        gameThread.start();
    }

    /**
     * Gibt die Auflösung in x-Richtung zurück.
     *
     * @return die Auflösung in x-Richtung
     */
    public int getScreenWidth() {
        return screenWidth;
    }

    /**
     * Gibt die Auflösung in y-Richtung zurück.
     *
     * @return die Auflösung in y-Richtung
     */
    public int getScreenHeight() {
        return screenHeight;
    }

}
